#pragma once

#include <cuda_runtime.h>

class CudaTimer
{
  public:

  CudaTimer() {
    cudaEventCreate(&_start);
    cudaEventCreate(&_stop);
    _time = 0.0f;
    _isRunning = false;
  }

  ~CudaTimer() {
    cudaEventDestroy( _start );
    cudaEventDestroy( _stop );
  }

  void start()
  {
    if (_isRunning) return;

    cudaEventRecord( _start, 0 );
    _isRunning = true;
  }
    
  void stop()
  {
    if (! _isRunning) return;

    cudaEventRecord( _stop, 0 );
    cudaEventSynchronize( _stop );
    float dt;
    cudaEventElapsedTime( &dt, _start, _stop );
    _time += dt;
    _isRunning = false;
  }

  double elapsed()
  {
    if (_isRunning) return -1.;
    return _time * 0.001;
  }

  private:
    cudaEvent_t _start, _stop;
    float _time;
    bool _isRunning;
};